package hexIT;

import javax.swing.*;
import java.awt.*;
import java.util.*;

/**
 *  A frame that contains a hex panel for displaying the board.
 */
public class HexFrame extends JFrame
{
   public static int TYPE_HEXAGONAL=      1;
   public static int TYPE_DIAMOND=        2;

   private int width, height, boardSize;
   private Color drawingColor= Color.gray;

	private HexPanel mainPanel;

	/**
	 *  Creates a new frame wigh the specified height and width. The type is
	 *  currently one of the supported type HexFrame.TYPE_HEXAGONAL,
	 *  or HexFrame.TYPE_DIAMOND. The boardsize is the size of the board in
	 *  the meaning designated by the board type. The board starts off hidden.
	 */
   public HexFrame(String frameName, int new_width, int new_height, int type,
	   int new_boardSize, Color new_bgColor)
   {
      super(frameName); 
      setVisible(false);         

      width= new_width;
      height= new_height;
      boardSize= new_boardSize;

      if (type == HexFrame.TYPE_HEXAGONAL)
         mainPanel= new HexagonalHexPanel(width, height, boardSize, new_bgColor, null);
      if (type == HexFrame.TYPE_DIAMOND)      
         mainPanel= new DiamondHexPanel(width, height, boardSize, new_bgColor);

      setBackgroundColor(Color.black);         
      setContentPane(mainPanel);         

      setSize(width, height);
   }

	/** Show the board grid. */
   public void showGrid()
   {
      mainPanel.showGrid(Color.gray);
   }

	/** Show the board, graphically. */
   public void showGrid(Color drawingColor)
   {
      mainPanel.showGrid(drawingColor);
   }

	/** Hide the board grid. */
   public void hideGrid()
   {
      mainPanel.hideGrid();
   }

	/** Set the background for the fram and panel. */
   public void setBackgroundColor(Color c)
   {
      setBackground(c);
      mainPanel.setBackground(c);
   }

	/** Set the drawing color of the hexagon grid. */
	public void setDrawingColor(Color new_color)
   {
      drawingColor= new_color;
   }

	/**
	 *  Put the specified object at hex location (x,y)
	 *  Note: the hex coords depend on the coordinate system used
	 *  by the board type.
	 */
   public void putObject(int x, int y, Drawable obj)
   {
      mainPanel.putObject(x, y, obj);
   }

	/**
	 *  Remove the specified object from hex location (x,y)
	 *  Note: the hex coords depend on the coordinate system used
	 *  by the board type.
	 */
   public void removeObject(int x, int y)
   {
      mainPanel.removeObject(x, y);
   }

	/** Return the main panel. */
   public HexPanel getHexPanel()
   {
      return mainPanel;
   }
         
}



