package hexIT;

import javax.swing.*;
import java.awt.*;

/**
 * A hex panel is a panel which contains a board of hexagons. The board type is
 * important to figuring out the graphical arrangement/layout of the component
 * hexagons, therefore this class only contains the shared methods that is part
 * of all kinds of hex panels.
 *
 * @author Marc Lanctot
 * @version 1.0
 * @last modified: Oct 6, 2003
 * @changes: comments added
 */
public abstract class HexPanel extends JPanel
{
   protected Board board;

   protected double  sin30= 0.5000000, 
                     cos30= 0.8660254;
                     
   protected int totalWidth, totalHeight, viewportWidth,
	              viewportHeight, topLeftX, topLeftY, sideLength;

   protected int boardSize;    // size of the board .. not pixels, in hexagons

   protected Color bgColor;
   protected boolean scale= false;

	/**
	 *  Create a new hex panel with the specified width heights and heights (in pixels),
	 *  and background color. The boardsize is different for every board type: it is the
	 *  size represented by the board type (see the specific board type that will be used).
	 */
	public HexPanel(int new_width, int new_height, int new_boardSize, Color new_bgColor)
   {
      totalWidth= new_width;
      totalHeight= new_height;
      boardSize= new_boardSize;
      bgColor= new_bgColor;

      // chop off 5% of the width & height
      setViewportWidth((int)((double)totalWidth - (0.1 * ((double)totalWidth))));
      setViewportHeight((int)((double)totalHeight - (0.1 * ((double)totalHeight))));
   }

   protected void setViewportHeight(int newHeight)
   {
      viewportHeight= newHeight;
      topLeftY= (totalHeight-viewportHeight)/2;
   }

   protected void setViewportWidth(int newWidth)
   {
      viewportWidth= newWidth;
      topLeftX= (totalWidth-viewportWidth)/2;
   }

	/** Gets the background color of this panel. */
   public Color getBackgroundColor()
   {
      return bgColor;
   }

	/** If the argument is true, scale all the images. */
   public void setScale(boolean new_scale)
   {
      boolean oldscale= scale;
      scale= new_scale;

      if (new_scale == true)
         scaleAllImages();
   }

	/** Sets the background color of this panel. */
   public void setBackgroundColor(Color new_bgColor)
   {
      bgColor= new_bgColor;
      repaint();
   }    
   
	/** Gets the board which is associated with this panel. */
   public Board getBoard()
   {
      return board;
   }

	/**
	 *  Puts the specified object at hex coordinates (x,y).
	 *  Note: the hex coordinates depends on the coordinate system
	 *        used by the actual board type.
	 */
   public abstract void putObject(int x, int y, Drawable object);

	/**
	 *  Removes the specified object at hex coordinates (x,y).
	 *  Note: the hex coordinates depends on the coordinate system
	 *        used by the actual board type.
	 */
	public abstract void removeObject(int x, int y);

	/** Shows the hex grid (all the hexagons on the board). */
   public abstract void showGrid(Color drawingColor);

	/** Hides the grid. All hexagons become not visible. */
	public abstract void hideGrid();

	/** Scale all the images of all the objects on the Hexagons. */
	public abstract void scaleAllImages();

	/**
	 *  Set the outline color of the hexagon at hex coords (hexx, hexy).
	 *  Note: the hex coordinates depends on the coordinate system
	 *        used by the actual board type.
 	 */
   public abstract void setHexOutlineColor(int hexx, int hexy, Color color);

	/**
	 *  Set the outline color of the hexagon at hex coords (hexx, hexy).
	 *  Note: the hex coordinates depends on the coordinate system
	 *        used by the actual board type.
 	 */
	public abstract void setHexFillColor(int hexx, int hexy, Color color);
}

