package hexIT;

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import javax.imageio.*;
import java.io.*;

/**
 *  ImageTools is a helper class that provides convenient methods
 *  for loading, caching, and scaling images.
 */
public class ImageTools
{
	private static HashMap cachedImages= new HashMap(30, (float)1.00);

	/**
	 *  Load the image with the specified filename and return it. Note, this
	 *  image is then cached in an image cache. Subsequent calls to this method
	 *  will return the cached image. If the file changes and an image with the
	 *  same name is loaded, this will not work.
	 */
   public static Image loadImage(String filename)
   {
		ImageIcon ii= null;
		if (cachedImages.containsKey(filename))
			return (Image)cachedImages.get(filename);
		
		try
		{
			File imageFile= new File(filename);
			BufferedImage bi= ImageIO.read(imageFile);
      	ii= new ImageIcon(bi);
      
      	if (ii.getImageLoadStatus() == MediaTracker.ERRORED)
      	{
         	System.out.println("Error loading image: " + filename);
         	return null;
      	}
      
      	// wait until the image is loaded
      	while (ii.getImageLoadStatus() != MediaTracker.COMPLETE)
      	{
         	System.out.println("Waiting for image: " + filename + "...");
         	try { Thread.sleep(10); } catch (Exception e) { e.printStackTrace(); System.exit(-1); }
      	}

			cachedImages.put(filename, ii.getImage());
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

      return ii.getImage();
   }

	/**
	 *  Scale the specified image. This uses the default scaling algorithm.
	 *  For more scaling functionality, see Image.getScaledInstance();
	 */
   public static Image scaleImage(Image img, int new_width, int new_height)
   {
      return img.getScaledInstance(new_width, new_height, Image.SCALE_DEFAULT);
   }

	/**
	 *  Scale the specified image. If it is cached, the cached copy is used.
	 *  This uses the default scaling algorithm.
	 *  For more scaling functionality, see Image.getScaledInstance();
	 */
   public static Image loadScaledImage(String filename, int new_width, int new_height)
   {
      return ImageTools.scaleImage(ImageTools.loadImage(filename), new_width, new_height);
   }
}
      

